/*
multiwrap.c - Generate a multipart/alternative envelope around a message
Copyright (C) 2003 Ian Gulliver

This program is free software; you can redistribute it and/or modify
it under the terms of version 2 of the GNU General Public License as
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include "firemake.h"

struct firestring_estr_t message;
struct firestring_estr_t subheaders = {0};
int mime = 1;
const char charset[] = "0123456789abcdef";

int foldback(struct firestring_estr_t *headers, struct firestring_estr_t *name) {
	struct firestring_estr_t *fullheader;

	fullheader = firemime_get_full_header(headers,name,&ESTR_S("\n"));
	if (fullheader == NULL)
		return -1;

	firestring_estr_aestrcat(&subheaders,fullheader,0);

	memmove(fullheader->s,fullheader->s + fullheader->l,headers->l - (fullheader->s - headers->s + fullheader->l));
	headers->l -= fullheader->l;

	return 0;
}

int handler(struct firemime_part *stack, int depth, void *opaque) {
	struct firestring_estr_t addheader = {0}, *tempheader, delim;
	int l;

	if (firestring_estr_estrcasecmp(&stack[depth].type,&ESTR_S("multipart"),0) == 0 &&
			firestring_estr_estrcasecmp(&stack[depth].subtype,&ESTR_S("alternative"),0) == 0) {
		write(1,message.s,message.l);
		exit(0);
	}

	if (foldback(&stack[depth].headers,&ESTR_S("Content-Type")) == -1)
		firestring_estr_aestrcat(&subheaders,&ESTR_S("Content-Type: text/plain\n"),0);
	foldback(&stack[depth].headers,&ESTR_S("Content-Transfer-Encoding"));
	foldback(&stack[depth].headers,&ESTR_S("Content-Dispostion"));
	foldback(&stack[depth].headers,&ESTR_S("Content-ID"));

	tempheader = firemime_get_header(&stack[depth].headers,&ESTR_S("MIME-Version"),&ESTR_S("\n"));
	if (tempheader == NULL)
		firestring_estr_aestrcat(&addheader,&ESTR_S("MIME-Version: 1.0\n"),0);

	srand(time(NULL));
	firestring_estr_alloc(&delim,32);
	for (l = 0; l < 32; l++)
		delim.s[l] = charset[rand() % 16];
	delim.l = 32;
	
	firestring_fprintf(stdout,"%e%eContent-Type: multipart/alternative; boundary=\"%e\"\n"
"\n"
"--%e\n"
"%e\n"
"%e\n"
"--%e--\n",&stack[depth].headers,&addheader,&delim,&delim,&subheaders,&stack[depth].body,&delim);

	exit(0);

	return 2;
}

int main(int argc, char *argv[]) {
	int l;
	int r;

	if (argc != 1 && !(argc == 2 && strcmp(argv[1],"--nomime") == 0)) {
		firestring_fprintf(stderr,"Usage: %s [--nomime]\n",argv[0]);
		exit(1);
	}

	if (argc == 2 && strcmp(argv[1],"--nomime") == 0)
		mime = 0;

	firestring_estr_alloc(&message,4096);
	while ((l = read(0,&message.s[message.l],message.a - message.l)) > 0) {
		message.l += l;
		firestring_estr_expand(&message,message.l + 4096);
	}

	/* we now have a message; find the end of the headers */

	r = firemime_parse(&message,handler,&ESTR_S("\n"),NULL);

	firestring_fprintf(stderr,"Error parsing MIME structure: %s\n",firemime_strerror(r));
	exit(1);
}
