/*
sendmailpay.c - Sendmail wrapper that calls firepay
Copyright (C) 2003 Ian Gulliver

This program is free software; you can redistribute it and/or modify
it under the terms of version 2 of the GNU General Public License as
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <sys/types.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include "firemake.h"

char *recips[256];
int r = 0;

#define CONF CONFDIR "/sendmailpay.conf"

char *loadconf(struct firestring_conf_t *conf, char *name) {
	char *t;
	t = firestring_conf_find(conf,name);
	if (t == NULL) {
		firestring_fprintf(stderr,"Error: %s not set in /etc/sendmailpay.conf\n",name);
		exit(1);
	}
	return t;
}

void foldback(struct firestring_estr_t *headers, struct firestring_estr_t *name) {
	struct firestring_estr_t *fullheader;

	fullheader = firemime_get_full_header(headers,name,&ESTR_S("\n"));
	if (fullheader == NULL)
		return;

	memmove(fullheader->s,fullheader->s + fullheader->l,headers->l - (fullheader->s - headers->s + fullheader->l));
	headers->l -= fullheader->l;
}

void addrecips(struct firestring_estr_t *headers, struct firestring_estr_t *name) {
	struct firestring_estr_t *header, localheader = {0};
	int l,s = 0,g = 1;

	header = firemime_get_header(headers,name,&ESTR_S("\n"));
	if (header == NULL)
		return;

	firestring_estr_aestrcpy(&localheader,header,0);

	while (g) {
		struct firestring_estr_t block;
		int j;

		l = firestring_estr_strchr(&localheader,',',s);
		if (l == -1) {
			l = localheader.l;
			g = 0;
		}

		block.s = &localheader.s[s];
		block.l = block.a = l - s;
		s = l + 1;

		/* we've got an address block in block */

		/* see if it's in brackets */
		j = firestring_estr_strchr(&block,'<',0);
		if (j != -1) {
			block.l = block.a = block.l - j - 1;
			block.s += j + 1;
			j = firestring_estr_strchr(&block,'>',0);
			if (j != -1)
				block.l = block.a = j;
		}

		firestring_estr_trim(&block);
		recips[r] = firestring_malloc(block.l + 1);
		memcpy(recips[r],block.s,block.l);
		recips[r++][block.l] = '\0';
	}
}

int handler(struct firemime_part *stack, int depth, void *opaque) {
	/* check to, cc and bcc headers for data */
	addrecips(&stack[depth].headers,&ESTR_S("To"));
	addrecips(&stack[depth].headers,&ESTR_S("CC"));
	addrecips(&stack[depth].headers,&ESTR_S("BCC"));

	/* don't leak bcc data */
	foldback(&stack[depth].headers,&ESTR_S("BCC"));

	return 2;
}

int main(int argc, char *argv[]) {
	int skiprecip = 0;
	char *outargs[256];
	int i,o = 1;
	struct firestring_conf_t *conf;
	char *real_sendmail, *multiwrap, *firepay, *amount, *temp_wait;
	struct firestring_estr_t message;
	int p1[2],p2[2];
	int l,j;
	int dowait = 0;
	int pullfromheader = 0;

	/* don't need standard output, apache cleanup */
	close(1);

	if (argc > 255) {
		firestring_fprintf(stderr,"Error: too many arguments.\n");
		exit(1);
	}

	conf = firestring_conf_parse(CONF);
	if (conf == NULL) {
		firestring_fprintf(stderr,"Error: unable to open/parse " CONF ": %s\n",strerror(errno));
		exit(1);
	}

	real_sendmail = loadconf(conf,"real_sendmail");
	firepay = loadconf(conf,"firepay");
	amount = loadconf(conf,"amount");
	multiwrap = firestring_conf_find(conf,"multiwrap");

	temp_wait = firestring_conf_find(conf,"wait");
	if (temp_wait != NULL && atoi(temp_wait) == 1)
		dowait = 1;

	outargs[0] = real_sendmail;

	for (i = 1; i < argc; i++) {
		if (argv[i][0] == '-') {
			if (strcmp(argv[i],"-t") == 0) {
				pullfromheader = 1;
				continue;
			}
			outargs[o++] = argv[i];
			if (strcmp(argv[i],"-f") == 0 || strcmp(argv[i],"-F") == 0)
				skiprecip++;
		} else if (skiprecip > 0) {
			outargs[o++] = argv[i];
			skiprecip--;
		} else
			recips[r++] = argv[i];
	}

	if (multiwrap != NULL) {
		if (pipe(p1) != 0) {
			firestring_fprintf(stderr,"Error: unable to pipe(): %s\n",strerror(errno));
			exit(1);
		}
		j = fork();
		if (j == -1) {
			firestring_fprintf(stderr,"Error: unable to fork(): %s\n",strerror(errno));
			exit(1);
		} else if (j != 0) {
			close(p1[1]);
			if (dup2(p1[0],0) != 0) {
				firestring_fprintf(stderr,"Error: uanble to dup2(): %s\n",strerror(errno));
				exit(1);
			}
		} else {
			close(p1[0]);
			if (dup2(p1[1],1) != 1) {
				firestring_fprintf(stderr,"Error: unable to dup2(): %s\n",strerror(errno));
				exit(1);
			}
			execl(multiwrap,multiwrap,NULL);
			firestring_fprintf(stderr,"Error: unable to execl(%s): %s\n",multiwrap,strerror(errno));
			exit(1);
		}
	}

	firestring_estr_alloc(&message,4096);
	while ((l = read(0,&message.s[message.l],message.a - message.l)) > 0) {
		message.l += l;
		firestring_estr_expand(&message,message.l + 4096);
	}
	close(0); /* piping is over folks, nothing to see here */

	if (pullfromheader) {
		int r;
		r = firemime_parse(&message,handler,&ESTR_S("\n"),NULL);
		if (r != 2) {
			firestring_fprintf(stderr,"-t flag passed and unable to parse message: %s\n",firemime_strerror(r));
			exit(1);
		}
	}

	for (i = 0; i < r; i++) {
		if (pipe(p1) != 0) {
			firestring_fprintf(stderr,"Error: unable to pipe(): %s\n",strerror(errno));
			exit(1);
		}
		if (pipe(p2) != 0) {
			firestring_fprintf(stderr,"Error: unable to pipe(): %s\n",strerror(errno));
			exit(1);
		}
		j = fork();
		if (j == -1) {
			firestring_fprintf(stderr,"Error: unable to fork(): %s\n",strerror(errno));
			exit(1);
		} else if (j != 0) {
			/* still in the main sendmailpay process */
			close(p1[0]);
			close(p2[1]);
			if (write(p1[1],message.s,message.l) != message.l) {
				firestring_fprintf(stderr,"Error: uanble to write(): %s\n",strerror(errno));
				exit(1);
			}
			close(p1[1]);

			/* now fork off sendmail while firepay is working */
			j = fork();
			if (j == -1) {
				firestring_fprintf(stderr,"Error: unable to fork(): %s\n",strerror(errno));
				exit(1);
			} else if (j != 0) {
				/* still in the main sendmailpay process */
				close(p2[0]);
				continue;
			}

			/* in the sendmail process */

			if (dup2(p2[0],0) != 0) {
				firestring_fprintf(stderr,"Error: unable to dup2(): %s\n",strerror(errno));
				exit(1);
			}

			outargs[o++] = recips[i];
			outargs[o] = NULL;
			execv(real_sendmail,outargs);
			firestring_fprintf(stderr,"Error: unable to execl(%s): %s\n",real_sendmail,strerror(errno));
			exit(1);
		}

		/* in the firepay process */

		close(p1[1]);
		close(p2[0]);
		if (dup2(p1[0],0) != 0) {
			firestring_fprintf(stderr,"Error: unable to dup2(): %s\n",strerror(errno));
			exit(1);
		}

		if (dup2(p2[1],1) != 1) {
			firestring_fprintf(stderr,"Error: unable to dup2(): %s\n",strerror(errno));
			exit(1);
		}

		/* input and output set up, now exec */
		execl(firepay,firepay,recips[i],amount,NULL);
		firestring_fprintf(stderr,"Error: unable to execl(%s): %s\n",firepay,strerror(errno));
		exit(1);
	}

	if (dowait == 1) {
		r *= 2;
		while (r > 0) {
			int status;
			if (wait(&status) == -1) {
				firestring_fprintf(stderr,"Error: unable to wait(): %s\n",strerror(errno));
				exit(1);
			}
			if (!WIFEXITED(status)) {
				firestring_fprintf(stderr,"Error: child did not exit normally\n");
				exit(1);
			}
			if (WEXITSTATUS(status) != 0) {
				firestring_fprintf(stderr,"Error: child exited with status %d\n",WEXITSTATUS(status));
				exit(1);
			}
			r--;
		}
	}

	return 0;
}
